# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2020, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Levi
# @version  1.0.0
# @brief    OSS数据管理
# @address  官网：      https://help.aliyun.com/document_detail/32026.html?spm=a2c6h.13066369.0.0.4c9f4424HfuqHI
#           GitHub:    https://github.com/aliyun/aliyun-oss-python-sdk?spm=a2c4g.11186623.2.9.4ddb585eouhim6
#
#  History:
#      <author>     <time>     <version>   <desc>
#       Levi      2010-06-04     1.0.0   OSS数据管理
#############################################################


__version__ = "1.0.0"


import os
import oss2

import logging
import threading


__all__ = [
    'init',
    'deinit',

    'upload',
    'download',

    'delete',

    'rename',
]


"""
# 常量
"""


"""
# 枚举
"""


"""
# 全局变量
"""
# 关键字
__key = 'LTAI4GFYNbcx9TTuesXSuERU'

# 秘钥
__secret = 'zLgNoWJMk3oK4OBOBQ6wBuglNhohml'

# 授权信息
__auth = None

# 区域信息
__endpoint = 'http://oss-cn-hangzhou.aliyuncs.com'

# 存储桶名称
__bucketName = 'h-rs'

# 日志管理对象
logger = logging.getLogger(__name__)


def init(options={}):
    """
    初始化
    :param options:
               key
               secret
               endpoint
               bucketName
               success
               fail
               complete
    :return:
    """
    global __auth
    global __key
    global __secret
    global __endpoint
    global __bucketName

    key = options.get('key')
    if key is not None:
        __key = key

    secret = options.get('sec')
    if secret is not None:
        __secret = secret

    endpoint = options.get('endpoint')
    if endpoint is not None:
        __endpoint = endpoint

    bucketName = options.get('bucketName')
    if bucketName is not None:
        __bucketName = bucketName

    # 构建授权信息
    __auth = oss2.Auth(__key, __secret)


def deinit(options={}):
    """
    退出
    :param options:
               success
               fail
               complete
    :return:
    """
    global __auth

    __auth = None


def upload(localPath, path, options={}):
    """
    上传文件
    :param localPath:    本地路径
    :param path:         文件路径
    :param options:
               endpoint
               bucketName
               success
               fail
               complete
    :return:
    """
    global __bucketName
    global __endpoint

    bucket = None
    endpoint = options.get('endpoint')
    bucketName = options.get('bucketName')
    if bucketName is not None:
        if endpoint is not None:
            bucket = __getBucket(bucketName, endpoint)
        else:
            bucket = __getBucket(bucketName, __endpoint)
    else:
        if endpoint is not None:
            bucket = __getBucket(__bucketName, endpoint)
        else:
            bucket = __getBucket(__bucketName, __endpoint)

    if bucket is not None:
        thread = threading.Thread(target=__uploadThread, args=(localPath, bucket, path, options))
        thread.start()
        logger.info("启动数据上传线程")


def uploadBatch(localDir: str, path: str, options={}):
    """
    批量上传
    :param localDir:   本地目录
    :param path:       上传路径，目录路径
    :param options:
               success
               fail
               complete
    :return:
    """
    if os.path.exists(localDir) is False:
        logger.error("本地目录不存在")
        options.get("fail") and options.get("fail")()
        options.get("complete") and options.get("complete")()
    else:
        result = True

        def fail():
            result = False

        files = os.listdir(localDir)
        for file in files:
            filePath = os.path.join(localDir, file)
            fileName = os.path.basename(filePath)

            storePath = None
            if path.endswith("/") is False:
                storePath = path + '/' + fileName
            else:
                storePath = path + fileName

            upload(filePath, storePath, {
                'fail': fail
            })

        if result is False:
            options.get("fail") and options.get("fail")()
            options.get("complete") and options.get("complete")()
        else:
            options.get("success") and options.get("success")()
            options.get("complete") and options.get("complete")()


def download(path, localPath, options={}):
    """
    下载文件
    :param path:         文件路径
    :param localPath:    本地文件路径
    :param options:
               endpoint
               bucketName
               success
               fail
               complete
    :return:
    """
    global __endpoint
    global __bucketName

    bucket = None
    endpoint = options.get('endpoint')
    bucketName = options.get('bucketName')
    if bucketName is not None:
        if endpoint is not None:
            bucket = __getBucket(bucketName, endpoint)
        else:
            bucket = __getBucket(bucketName, __endpoint)
    else:
        if endpoint is not None:
            bucket = __getBucket(__bucketName, endpoint)
        else:
            bucket = __getBucket(__bucketName, __endpoint)

    if bucket is not None:
        thread = threading.Thread(target=__downloadThread, args=(bucket, path, localPath, options))
        thread.start()
        logger.info("启动数据下载线程")


def delete(path, options={}):
    """
    删除文件
    :param path:           文件路径
    :param options:
               endpoint
               bucketName
               success
               fail
               complete
    :return:
    """
    global __endpoint
    global __bucketName

    bucket = None
    endpoint = options.get('endpoint')
    bucketName = options.get('bucketName')
    if bucketName is not None:
        if endpoint is not None:
            bucket = __getBucket(bucketName, endpoint)
        else:
            bucket = __getBucket(bucketName, __endpoint)
    else:
        if endpoint is not None:
            bucket = __getBucket(__bucketName, endpoint)
        else:
            bucket = __getBucket(__bucketName, __endpoint)

    if bucket is not None:
        bucket.delete_object(path)
        options.get('success') and options.get('success')()
        options.get('complete') and options.get('complete')()
    else:
        options.get('fail') and options.get('fail')()
        options.get('complete') and options.get('complete')()


def rename(path, newName, options={}):
    """
    重命名文件
    :param path:          文件路径
    :param newName:       新文件名
    :param options:
               success
               fail
               complete
    :return:
    """
    oldName = os.path.basename(path)
    if oldName == newName:
        logger.info("原文件和新文件名称相同")
        return

    # 获取存储桶对象
    if __bucketName is not None:
        bucket = __getBucket(__bucketName)
        if bucket is not None:
            # 获取主目录路径，构造新文件名称
            dir = os.path.dirname(path)
            if dir.endswith('/') is False:
                dir = dir + '/'
            newPath = dir + newName

            # 执行拷贝，拷贝成功后删除原文件
            result = bucket.copy_object(__bucketName, path, newPath)
            if result.status == 200:
                delete(path, options)
            else:
                logger.error("文件新文件失败")


def listFiles(prefix, isfile=False, options={}):
    """
    查询文件列表
    :param prefix:       前缀
    :param options:
               success
               fail
               complete
    :return:   objects   文件列表
    """
    objects = []
    if __bucketName is not None:
        bucket = __getBucket(__bucketName)
        if bucket is not None:
            for obj in oss2.ObjectIterator(bucket, prefix=prefix, delimiter='/'):
                if isfile is True:
                    if obj.is_prefix() is False:
                        objects.append(obj)
                else:
                    objects.append(obj)
        else:
            logger.info("获取bucket失败")
    return objects


def __getBucket(bucketName, endpoint=None, options={}):
    """
    获取存储桶对象
    :param bucketName:   存储桶名称
    :param options:
               success
               fail
               complete
    :return:
    """
    global __auth
    global __endpoint
    if endpoint is not None:
        bucket = oss2.Bucket(__auth, endpoint, bucketName)
    else:
        bucket = oss2.Bucket(__auth, __endpoint, bucketName)

    return bucket


def __downloadThread(bucket, path, localPath, options):
    """
    数据下载线程
    :param bucket:     存储桶对象
    :param path:       存储路径
    :param localPath:  本地文件路径
    :param options:
               success
               fail
               complete
    :return:
    """
    flag = True
    dir = os.path.dirname(localPath)
    if os.path.exists(dir) is False:
        os.makedirs(dir)
    try:
        # TODO:下载的时候，用断点续传，总是会报错，暂时屏蔽，get_object_to_file
        bucket.get_object_to_file(key=path, filename=localPath)
        # oss2.resumable_download(bucket=bucket, key=path, filename=localPath)
    except Exception as err:
        if len(err.args) > 0:
            if err.args[0] != '504':
                try:
                    bucket.get_object_to_file(key=path, filename=localPath)
                    # oss2.resumable_download(bucket=bucket, key=path, filename=localPath)
                except Exception as err2:
                    flag = False
                    logger.error(err2)
                    options.get('fail') and options.get('fail')()
                    options.get('complete') and options.get('complete')()
        else:
            flag = False
            logger.error(err)
            options.get('fail') and options.get('fail')()
            options.get('complete') and options.get('complete')()
    if flag is True:
        options.get('success') and options.get('success')({
            'path': path,
            'localPath': localPath
        })
        options.get('complete') and options.get('complete')()


def __uploadThread(localPath, bucket, path, options):
    """
    数据上传线程
    :param localPath:   本地文件路径
    :param bucket:      存储对象
    :param path:        存储路径
    :param options:
               success
               fail
               complete
    :return:
    """
    flag = True
    try:
        oss2.resumable_upload(bucket=bucket, key=path, filename=localPath)
    except Exception as err:
        if len(err.args) > 0:
            if err.args[0] != '504':
                try:
                    oss2.resumable_upload(bucket=bucket, key=path, filename=localPath)
                except Exception as err2:
                    flag = False
                    logger.error(err2)
                    options.get('fail') and options.get('fail')()
                    options.get('complete') and options.get('complete')()
        else:
            flag = False
            logger.error(err)
            options.get('fail') and options.get('fail')()
            options.get('complete') and options.get('complete')()
    if flag is True:
        options.get('success') and options.get('success')({
            'localPath': localPath,
            'path': path
        })
        options.get('complete') and options.get('complete')()
