# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2020, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Levi
# @version  1.0.0
# @brief    模板模块
#  History:
#      <author>     <time>     <version>   <desc>
#       Levi     2020-04-03     1.0.0      配置模块
#############################################################
import time
import configparser


__all__ = [
    'config',
    'getConfigPath',
    'getVal',
    'getSecs',
    'getOptins',
    'getItems',
    'addSec',
    'write',
    'writeWithTime',
]


# 配置文件路径
__configPath = 'D:\\UploadFiles\\hre\\hre.config'

# 配置解析器
__config = configparser.ConfigParser()


def config(configPath):
    """
    设置配置路径
    :param configPath: 配置文件路径
    :return:
    """
    global __configPath
    __configPath = configPath
    __config.read(__configPath)


def getConfigPath():
    """
    获取配置路径
    :return: 配置文件路径
    """
    global __configPath
    return __configPath


def getVal(section, option):
    """
    获取配置值
    :param section: 区域
    :param option:  选择项
    :return:       配置值
    """
    global __config
    return __config.get(section, option)


def getSecs():
    """
    获取所有区域
    :return: 区域列表
    """
    global __config
    return __config.sections()


def getOptins(section):
    """
    获取所有选择项
    :param section: 区域
    :return:        选择项列表
    """
    global __config
    return __config.options(section)


def getItems(section):
    """
    获取所有要素
    :param section: 区域
    :return:        要素列表
    """
    global __config
    return __config.items(section)


def addSec(section):
    """
    添加区域
    :param section: 区域
    :return:
    """
    global __config

    if __config.has_section(section) is False:
        __config.add_section(section)


def write(sec, key, val):
    """
    写入区域信息
    :param sec: 区域
    :param key: 关键字
    :param val: 配置值
    :return:
    """
    global __config
    global __configPath

    __config.read(__configPath)
    if __config.has_section(sec) is False:
        __config.add_section(sec)
    __config.set(sec, key, val)
    with open(__configPath, "w+") as file:
        __config.write(file)


def writeWithTime(sec, key, val):
    """
    写入带时间串区域信息
    :param sec:   区域
    :param key:   关键字
    :param val:   配置值
    :return:
    """
    global __config
    global __configPath

    __config.read(__configPath)
    if __config.has_section(sec) is False:
        __config.add_section(sec)

    # 写入内容
    curTimeStr = time.strftime('%Y-%m-%d %H:%M:%S',time.localtime(time.time()))
    val = 'time: ' + curTimeStr + ', val: ' + val

    __config.set(sec, key, val)
    with open(__configPath, "w+") as file:
        __config.write(file)


