# !usr/bin/env python
# -*- coding: UTF-8 -*-

#############################################################
#        Copyright (c), 2015-2019, 武汉禾大科技有限公司
#                    All Rights Reserved
# @author   Levi
# @version  1.0.0
# @brief    MYSQL数据库模块
#           https://kazoo.readthedocs.io/en/latest/
#  History:
#      <author>     <time>     <version>   <desc>
#       Levi     2020-07-14     1.0.0    MYSQL数据库
#############################################################
import logging
import pymysql

__version__ = "1.0.0"


__all__ = [
    'connect',
    'close',
    'getConn',
    'selectdb',
    'execute',
    'insert',
    'query',
    'queryByWhere',
    'delete'
]


"""
# 常量
"""


"""
# 枚举
"""


"""
# 全局变量
"""
# 日志对象
logger = logging.getLogger(__name__)

# 连接信息
__connectInfo = {
    'host': 'localhost',
    'port': 3306,
    'user': 'root',
    'password': 'root',
    'database': ''
}

# 连接对象列表
__connects = [
    # 'id': ''
    # 'conn': None
]


def connect(id, options={}):
    """
    连接数据库
    :param id:       主键
    :param options:
               host
               port
               user
               password
               database
               charset
    :return:
    """
    global __connectInfo

    result = None
    connect = __connectInfo

    conn = getConn(id)
    if conn is not None:
        if conn.open is True:
            result = conn
        else:
           conn.connect()
    else:
        host = options.get('host')
        if host is not None:
            connect['host'] = host
        port = options.get('port')
        if port is not None:
            connect['port'] = port
        user = options.get('user')
        if user is not None:
            connect['user'] = user
        password = options.get('password')
        if password is not None:
            connect['password'] = password
        database = options.get('database')
        if database is not None:
            connect['database'] = database
        result = pymysql.connect(host=connect['host'], port=connect['port'],
                                 user=connect['user'], password=connect['password'],
                                 database=connect['database'])
        __connects.append({
            'id': id,
            'conn': result
        })

    return result


def close(id, options={}):
    """
    关闭连接
    :param id:  主键
    :param options
               success
               fail
               complete
    :return:
    """
    global __connects
    conn = getConn(id)
    if conn is not None:
        conn.close()
        for item in __connects:
            if item.get('id') == id:
                __connects.remove(item)
                break
    else:
        logger.info("未找到id={}对应的连接".format(id))


def getConn(id):
    """
    获取连接
    :param id:   主键
    :return:     连接对象
    """
    global __connects

    conn = None
    for item in __connects:
        idCur = item.get('id')
        if id == idCur:
            conn = item.get('conn')
            break
    return conn


def selectdb(id, db):
    """
    选择数据库
    :param id:   主键
    :param db:   数据库名称
    :return:
    """
    conn = getConn(id)
    if conn is not None:
        conn.select_db(db)
    else:
        logger.info("未找到id={}对应的连接".format(id))


def execute(id, sql, options={}):
    """
    执行sql
    :param id:   主键
    :param sql:  sql语句
    :param options
           success
           fail
           complete
    :return:
    """
    conn = getConn(id)
    if conn is not None:
        cursor = conn.cursor()
        if cursor is not None:
            conn.ping(reconnect=True)
            cursor.execute(sql)
            conn.commit()
            cursor.close()
            options.get('success') and options.get('success')()
            options.get('complete') and options.get('complete')()
    else:
        logger.info("未找到id={}对应的连接".format(id))
        options.get('fail') and options.get('fail')()
        options.get('complete') and options.get('complete')()


def insert(id, table, dic, options={}):
    """
    插入表格数据
    :param id:       主键
    :param table:    表名
    :param dic:      字典
    :param options:
               success
               fail
               complete
    :return:
    """
    conn = getConn(id)
    if conn is not None:
        # 构造插入语句
        sql = 'insert into {} '.format(table)
        if len(dic) > 0:
            keySql = ' ('
            valSql = ' values('
            for key in dic.keys():
                keySql = keySql + key + ','
                if __typeof(dic[key]) == 'str':
                    # 处理斜杠的问题
                    dic[key] = dic[key].replace('\\', '\\\\')
                    valSql = valSql + "'" + dic[key] + "'" + ','
                else:
                    valSql = valSql + str(dic[key]) + ','
            keySql = keySql[:-1] + ')'
            valSql = valSql[:-1] + ')'
            sql = sql + keySql + valSql

            # 执行插入
            cursor = conn.cursor()
            conn.ping(reconnect=True)
            cursor.execute(sql)
            conn.commit()
            cursor.close()
            options.get('success') and options.get('success')()
            options.get('complete') and options.get('complete')()
        else:
            logger.info("字典为空，不做插入")
            options.get('fail') and options.get('fail')()
            options.get('complete') and options.get('complete')()
    else:
        logger.info("未找到id={}对应的连接".format(id))
        options.get('fail') and options.get('fail')()
        options.get('complete') and options.get('complete')()


def query(id, sql, type='dict', options={}):
    """
    查询数据
    :param id:        主键
    :param sql:       查询语句
    :param type:
               dict：
                   {'fields': ['code', 'name', 'county'],
                   'rows': (('150802100201', '团结村', '临河区'),
                   ('150802100224', '团结村', '临河区'))}
               value:
                   (('150802100201', '团结村', '临河区'),
                   ('150802100224', '团结村', '临河区'))
    :param options:
               success
               fail
               complete
    :return:   rows  数据结果字典或元组
    """
    rows = None
    conn = getConn(id)
    if conn is not None:
        cursor = conn.cursor()
        conn.ping(reconnect=True)
        cursor.execute(sql)

        if type == 'value':
            rows = cursor.fetchall()
        elif type == 'dict':
            cols = []
            rows = dict()
            for field in cursor.description:
                cols.append(field[0])
            rows['fields'] = cols
            rows['rows'] = cursor.fetchall()

        cursor.close()
        options.get('success') and options.get('success')()
        options.get('complete') and options.get('complete')()
    else:
        logger.info("未找到id={}对应的连接".format(id))
        options.get('fail') and options.get('fail')()
        options.get('complete') and options.get('complete')()

    return rows


def queryByWhere(id, table, whereDic, fields=None, type='dict', options={}):
    """
    根据where条件查询
    :param id:        主键
    :param table:     表名
    :param whereDic:  where条件字典
    :param fields:    需要查询的字段，None表示查所有（*）
    :param type:
               dict：
                   {'fields': ['code', 'name', 'county'],
                   'rows': (('150802100201', '团结村', '临河区'),
                   ('150802100224', '团结村', '临河区'))}
               value:
                   (('150802100201', '团结村', '临河区'),
                   ('150802100224', '团结村', '临河区'))
    :param options:
               success
               fail
               complete
    :return:   rows  数据结果字典或元组
    """
    sql = 'SELECT '
    if fields is None:
        sql = sql + '* FROM ' + table + ' where '
    else:
        for field in fields:
            sql = sql + field + ','
        sql = sql[:-1] + ' from ' + table + ' where '

    if len(whereDic) > 0:
        for key in whereDic.keys():
            if __typeof(whereDic[key]) == 'str':
                sql = sql + (key + '=' + "'" + whereDic[key] + "'" + ' and ')
            else:
                sql = sql + (key + '=' + str(whereDic[key]) + ' and ')
        sql = sql[:-4]

    rows = query(id, sql, type, options)
    return rows


def delete(id, table, whereDic, options={}):
    """
    条件删除
    :param id:             主键
    :param table:          表名
    :param whereDic:       条件字典
    :param options:
               success
               fail
               complete
    :return:
    """
    sql = 'delete from ' + table + ' where '
    if len(whereDic) > 0:
        for key in whereDic.keys():
            if __typeof(whereDic[key]) == 'str':
                sql = sql + (key + '=' + "'" + whereDic[key] + "'" + ' and ')
            else:
                sql = sql + (key + '=' + str(whereDic[key]) + ' and ')
        sql = sql[:-4]
    execute(id, sql, options)


def modify(id, table, dic, whereDic, options={}):
    """
    修改表中值
    :param id:          主键
    :param table:       表名称
    :param dic:         设置字典
    :param whereDic:    条件字典
    :param options:
               success
               fail
               complete
    :return:
    """
    sql = 'update ' + table + ' set '
    if len(dic) > 0:
        for key in dic.keys():
            if __typeof(dic[key]) == 'str':
                sql = sql + key + "='" + dic[key] + "',"
            else:
                sql = sql + key + "=" + str(dic[key]) + ","
        sql = sql[:-1]

    if len(whereDic) > 0:
        sql = sql + ' where '
        for whereKey in whereDic.keys():
            if __typeof(whereDic[whereKey]) == 'str':
                sql = sql + (whereKey + '=' + "'" + whereDic[whereKey] + "'" + ' and ')
            else:
                sql = sql + (whereKey + '=' + str(whereDic[whereKey]) + ' and ')
        sql = sql[:-4]

    conn = getConn(id)
    if conn is not None:
        cursor = conn.cursor()
        if cursor is not None:
            conn.ping(reconnect=True)
            cursor.execute(sql)
            conn.commit()
            cursor.close()
            options.get('success') and options.get('success')()
            options.get('complete') and options.get('complete')()
    else:
        logger.info("获取连接为空,修改失败,id={}".format(id))
        options.get('fail') and options.get('fail')()
        options.get('complete') and options.get('complete')()


def __typeof(variate):
    """
    数据类型
    :param variate: 变量
    :return:        类型
    """
    type = None
    if isinstance(variate, int):
        type = "int"
    elif isinstance(variate, str):
        type = "str"
    elif isinstance(variate, float):
        type = "float"

    return type

